#include <stdint.h>

#include <LPC214x.h>
#include "type.h"
#include "irq.h"
#include "timer.h"

#define	FOSC	    12000000        // Quarz Freq.
#define PLL_M	    5               // CPU Takt Multiplikator
#define PLL_P	    2               // Teiler für F_CC0 (muss zwischen 156 - 320MHz liegen)
#define CCLK	    (PLL_M * FOSC)  // CPU Takt als Zahlenwert
#define VPBDIV_VAL  1               // Teiler des Peripherie-Takts

#define LED_1       (10)
#define LED_2       (11)
#define BUT_1       (15)
#define BUT_2       (16)



void INIT_PLL(void);
void INIT_UART0(uint32_t baud);
void __attribute__ ((interrupt("IRQ"))) isr_UART0 (void);

void INIT_TIMER0(void);
void __attribute__ ((interrupt("IRQ"))) isr_TIMER0 (void);

void INIT_SPURIOS(void);
void __attribute__ ((interrupt("IRQ"))) isr_SPURIOS (void);

void put0c(char c);
void put0s(char* s);



void INIT_PLL(void)
{
    PLLCFG = (PLL_M-1)|((PLL_P-1)<<5);
    // CPU Takt = PLL_M * FOSC = 5 * 12MHz = 60MHz
    // CCO Takt = 2 * PLL_P * PLL_M * FOSC = 2 * 2 * 5 * 12MHz = 240MHz
    PLLCON = 0x01;  // PLL aktivieren
    PLLFEED = 0xAA; // PLL Feed-Sequenz
    PLLFEED = 0x55;
    //while (!(PLLSTAT & PLOCK)) ; //Warten bis übernommen wurde
    while((PLLSTAT & (1UL<<10)) == 0);//Warten bis übernommen wurde

    // Mit MAMTIM werden die Waitstates beim Flashspeicherzugriff eingestellt, das Datenblatt empfiehlt folgende Werte:
    // 1 - bei unter 20 Mhz
    // 2 - bei 20-40 Mhz
    // 3 - bei über 40 Mhz
    MAMCR = 0;      // MAM aus
    MAMTIM = 3;     // MAM fetch cycle to 3 cclk (>40MHz)
    MAMCR = 2;      // MAM vollständig aktiviert

    VPBDIV = VPBDIV_VAL;    // Prozessor-Takt=Peripherie-Takt

    PLLCON = 0x03;  // PLL aktivieren und mit dem internen Taktgeber verbinden
    PLLFEED = 0xAA; // PLL Feed Sequence
    PLLFEED = 0x55;
}

void INIT_UART0(uint32_t baud)
{
    PINSEL0 &= ~((0x3<<2) | (0x3<<0)); // Pin-Funktion löschen (zur Sicherheit)
    //         rxd 0     txd 0
    PINSEL0 |= (1<<2) | (1<<0);        // Pin-unktion zuweisen

    // BAUD RATE EINSTELLEN
    // Divisor Latch Access Bit setzen (DLAB)
    // damit erhalten wir Zugriff auf den Baudraten-Teiler
    U0LCR |= (1<<7);

    // Nun haben wir Zugriff auf DLL (untere 8 bit des Teilers)
    // und DLM (obere 8 bit des Teilers). Auffällig ist hier,
    // dass der 16-Bit-Wert auf zwei 32-Bit-Addressen aufgeteilt ist

    // Teiler = PCLK / (baudrate * 16)
    // Im Beispiel haben wir CCLK und PCLK gleich gesetzt
    // Untere 8 Bit des Teilers
    U0DLL = (CCLK / (baud*16)) & 0xFF;
    // Obere 8 Bit des Teilers
    U0DLM = ((CCLK / (baud*16)) & 0xFF00)>>8;

    // DLAB wieder löschen
    U0LCR &= ~(1<<7);

    // 9 bit, 1 stop bit, keine parität
    U0LCR = (1<<1) | 1;

    // UART0-FIFO aktivieren
    U0FCR = 1;
    
    //Receive Interrupt
    U0IER = 0x1;
    // Kanal 4 konfigurieren
    VICVectAddr4 = (unsigned long) isr_UART0;
    VICVectCntl4 = IRQ_SLOT_EN | UART0_INT;
    VICIntEnable = VICIntEnable | (1<<UART0_INT);
}

void __attribute__ ((interrupt("IRQ"))) isr_UART0 (void)
{
  int itemp;
  int buffer;
  // Read U1IIR and check if it is a CTI or RDA interrupt by checking bits 3:1
  // in the UART0 Interrupt Identification Register (U0IIR)
  itemp=U0IIR;
  itemp=(itemp>>1)& 0x7;
  // If CTI interrupt then toggle port pin 13 and continue with regular
  // operation
  if (itemp==6)
  {
    IOCLR0=(1<<LED_2);
    IOSET0=(1<<LED_2);
  }
  // If RDA interrupt then read the Rx FIFO and toggle port pin 4
  if (itemp==2)
  {
    //Reading the FIFO
    while (U0LSR & 0x01)
    {
      // The data is not important for this application
      // so it is simply read into a variable
      buffer=U0RBR;
      put0c((char)buffer);
    }
    IOCLR0=(1<<LED_2);
    IOSET0=(1<<LED_2);
  }
  // Updating the VIC
  VICVectAddr=0xFF;
}

void INIT_TIMER0(void)
{
  // Timer 0 Configuration
  T0TCR=0x0;
  T0TC=0x0;
  T0PR=0x0;
  T0PC=0x0;
  // Setting the Match value
  T0MR0=0xffff;
  // Reset and interrupt on match
  T0MCR=0x3;

  // Kanal 3 konfigurieren
  VICVectAddr1 = (unsigned long) isr_TIMER0;
  VICVectCntl1 = IRQ_SLOT_EN | TIMER0_INT;
  VICIntEnable = VICIntEnable | (1<<TIMER0_INT);
}

void __attribute__ ((interrupt("IRQ"))) isr_TIMER0 (void)
{
  IOSET0 |= (1<<LED_2);
//  timertick++;
//  put0c(timertick);
  IOCLR0 |= (1<<LED_2);

  T0IR = 1;          // Clear interrupt flag
  VICVectAddr = 0x0; // Acknowledge Interrupt
}

void INIT_SPURIOS(void)
{
  VICDefVectAddr=(unsigned long)isr_SPURIOS;
}

void __attribute__ ((interrupt("IRQ"))) isr_SPURIOS (void)
{
  int buf;
  
  IOCLR0 = (1<<LED_1);
  IOSET0 = (1<<LED_1);
  
  //Reading the FIFO
  while (U0LSR & 0x01)
  {
    buf = U0RBR;
  }

  T0IR=0x1;
  VICVectAddr=0xff; 
}

void put0c(char c)
{
    while (!(U0LSR & (1<<5))) continue; // Warten, bis der Sendepuffer geleert ist

    // Neuen Wert in das "Transmitter Holding Register" schreiben
    U0THR = c;
}

void put0s(char* s)
{
    while(*s)
    {
        put0c(*s);
        s++;
    }
}

int main(void)
{
    INIT_PLL();

    IOSET0 = (1<<LED_1);
    IODIR0 |= (1<<LED_1);

    IOSET0 = (1<<LED_2);
    IODIR0 |= (1<<LED_2);

    IODIR0 &= ~(1<<BUT_1);
    IODIR0 &= ~(1<<BUT_2);

    INIT_UART0(19200);
    INIT_SPURIOS();

    put0s("\n\r- START -\n\r");

    INIT_TIMER0();

    while (1)
    {
        if(IOPIN0 & (1<<BUT_1))
        {
            IOSET0 |= (1<<LED_1);
            //put0c('1');
        }
        else
        {
            //put0c('1');
            IOCLR0 |= (1<<LED_1);
        }
    }
    return 0;
}





















